/* 2013-07-16: File added and changed by Sony Corporation */
/* icd_board_type.c
 *
 * Copyright 2012 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <common.h>

#if defined(CONFIG_ICD)

#include "reg.h"
#include "icx_io.h"
#include "icx_gpio.h"
#include "icd_board_type.h"

#define INPUT_OPEN       0x44444444
#define INPUT_PULLUP     0x55555555
#define INPUT_PULLDOWN   0x77777777
#define INPUT_SCHMITT_OPEN	0xeeeeeeee

#define CHG_DRIVE1_SD_SDI0_MASK 0xf0003c00 // SD, SDI0
#define CHG_DRIVE1_CLK4_CMD8    0x80002000 // CLK 4mA, CMD 8mA
#define CHG_DRIVE1_CLK8_CMD8    0xa0002800 // CLK 8mA, CMD 8mA

/* board id GPIOs */
enum {
    BID_PIN_1 = 0,
    BID_PIN_2,
#ifdef CONFIG_ICD_0426
    BID_PIN_GPIO_50,
#endif
    BID_PIN_MAX
};

struct bid_pin_info {
    u32 gpio;
    u32 pull_reg;
    u32 pull_mask;
};

static struct bid_pin_info bid_pin[BID_PIN_MAX] = {
    { GPIO_P015, CHG_PULL1, 0xf0000000 },   // BID_PIN_1
    { GPIO_P019, CHG_PULL2, 0x0000f000 },   // BID_PIN_2
#ifdef CONFIG_ICD_0426
    { GPIO_P050, CHG_PULL6, 0x00000f00 },   // BID_PIN_GPIO_50
#endif
};

static int s_board_type = ICD_BOARD_TYPE_UNKNOWN;

#define PIN_CHECK(pin)  { if(pin < BID_PIN_1 || BID_PIN_MAX <= pin) return -1; }

/*-----------------------------------------------------------------------------
 * Function	: get_gpio_with_pull
 * Abstruct : set pull up/down then read gpio value
 * Input    : pin	: gpio pin
 *			: pull	: pull up/down/none
 * Return   : gpio value
 *----------------------------------------------------------------------------*/
static int set_gpio_pull(int pin, int pull)
{
    u32 reg;
    u32 mask;
    
    PIN_CHECK(pin);
    
    reg  = bid_pin[pin].pull_reg;
    mask = bid_pin[pin].pull_mask;
    
    outl((inl(reg) & ~mask) | (pull & mask), reg);
    return 0;
}

/*-----------------------------------------------------------------------------
 * Function	: get_gpio_with_pull
 * Abstruct : set pull up/down then read gpio value
 * Input    : pin	: gpio pin
 *			: pull	: pull up/down/none
 * Return   : gpio value
 *----------------------------------------------------------------------------*/
static int get_gpio_with_pull(int pin, int pull)
{
    PIN_CHECK(pin);
    
    set_gpio_pull(pin, pull);
    udelay(100);
    return gpio_get_value(bid_pin[pin].gpio);
}

/*-----------------------------------------------------------------------------
 * Function	: icd_board_type_init
 * Abstruct : initialize and detect board type
 * Return   : none
 *----------------------------------------------------------------------------*/
static void icd_board_config(int type)
{
    u32 reg = inl(CHG_DRIVE1);
    
    reg &= ~CHG_DRIVE1_SD_SDI0_MASK; // clear bits
    
    if(type & ICD_BOARD_TYPE_SET) {
        /* set */
        reg |= CHG_DRIVE1_CLK4_CMD8;
        //printf("[drv] set 0x%08x\n", reg);
    }
    else {
        /* BB */
        reg |= CHG_DRIVE1_CLK8_CMD8;
        //printf("[drv] bb 0x%08x\n", reg);
    }
    
    outl(reg, CHG_DRIVE1);
}

/*-----------------------------------------------------------------------------
 * Function	: icd_board_type_init
 * Abstruct : initialize and detect board type
 * Return   : none
 *----------------------------------------------------------------------------*/
void icd_board_type_init(void)
{
    int i, bid[BID_PIN_MAX];
    
    for(i=BID_PIN_1; i<BID_PIN_MAX; i++) {
        gpio_set_direction(bid_pin[i].gpio, GPIO_IN);
    }
    
    // default value
    s_board_type = ICD_BOARD_TYPE_UNKNOWN;
    
#ifdef CONFIG_ICD_0426
    if(get_gpio_with_pull(BID_PIN_GPIO_50, INPUT_PULLDOWN)) {
        printf("\n*** board type BB ***\n");
        s_board_type = ICD_BOARD_TYPE_BB;
    }
    else {
        if(get_gpio_with_pull(BID_PIN_1, INPUT_PULLUP)) {
            if(get_gpio_with_pull(BID_PIN_2, INPUT_OPEN)) {
                // SET
                printf("\n*** board type SET ***\n");
                s_board_type = ICD_BOARD_TYPE_SET;
            }
            else {
                // SET DEBUG
                printf("\n*** board type SET DEBUG ***\n");
                s_board_type = ICD_BOARD_TYPE_SET_DBG;
            }
        }
        else {
            // SET CHECKER
            printf("\n*** board type SET CHECKER ***\n");
            s_board_type = ICD_BOARD_TYPE_SET_CHK;
        }
    }
#else
    if(get_gpio_with_pull(BID_PIN_1, INPUT_OPEN)) {
        // PU -> BB or SET_DBG
        if(get_gpio_with_pull(BID_PIN_1, INPUT_PULLDOWN)) {
            // PU(1k) -> SET_DBG
            s_board_type = ICD_BOARD_TYPE_SET_DBG;
        }
        else {
            // PU(470k) -> BB
            s_board_type = ICD_BOARD_TYPE_BB;
        }
    }
    else {
        // PD -> SET or SET_CHK
        if(get_gpio_with_pull(BID_PIN_1, INPUT_PULLUP)) {
            // PD(470k) -> SET
            s_board_type = ICD_BOARD_TYPE_SET;
        }
        else {
            // PD(1k) -> SET_CHK
            s_board_type = ICD_BOARD_TYPE_SET_CHK;
        }
    }
#endif

    if(s_board_type == ICD_BOARD_TYPE_UNKNOWN) {
        printf("\n*** board type unknown, treat as BB ***\n");
        s_board_type = ICD_BOARD_TYPE_BB;
    }
    
    // board depend setting (eg. drive strength)
    icd_board_config(s_board_type);
    
#ifdef CONFIG_ICD_0426
    // set pull for stable state
    set_gpio_pull(BID_PIN_1, INPUT_PULLDOWN);
    set_gpio_pull(BID_PIN_2, INPUT_OPEN);
    if(s_board_type == ICD_BOARD_TYPE_BB) {
        // BB
        printf("*** GPIO_050=HiZ, GPIO_015=PD, GPIO_019=HiZ ***\n");
        set_gpio_pull(BID_PIN_GPIO_50, INPUT_OPEN);
    }
    else {
        // SET
        printf("*** GPIO_050=PD, GPIO_015=PD, GPIO_019=HiZ ***\n");
        set_gpio_pull(BID_PIN_GPIO_50, INPUT_PULLDOWN);
    }
#else
    // set pull for stable state
	set_gpio_pull(BID_PIN_1, INPUT_SCHMITT_OPEN);
	set_gpio_pull(BID_PIN_2, INPUT_SCHMITT_OPEN);
#endif
}

/*-----------------------------------------------------------------------------
 * Function	: icd_get_board_type
 * Abstruct : get board type
 * Return   : board type (refer icd hw spec)
 *----------------------------------------------------------------------------*/
int icd_get_board_type(void)
{
    return s_board_type;
}

/*-----------------------------------------------------------------------------
 * Function	: icd_get_board_type_string
 * Abstruct : board type id -> board type string
 * Input    : type	: board type id
 * Return   : board type string
 *----------------------------------------------------------------------------*/
char* icd_get_board_type_string(int type)
{
    switch(type) {
        case ICD_BOARD_TYPE_BB:
            return "BB";
        case ICD_BOARD_TYPE_SET:
            return "SET";
        case ICD_BOARD_TYPE_SET_DBG:
            return "SET DEBUG";
        case ICD_BOARD_TYPE_SET_CHK:
            return "SET CHECKER";
        default:
            return "unknown";
    }
}

#endif
