/* 2013-07-16: File added and changed by Sony Corporation */
/* icd_emmc_test.c
 *
 * Copyright 2012 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

#include <config.h>
#include <common.h>

#include "mini-boot/include/mmc.h"
#include "mini-boot/include/mini_config.h"
#include "reg.h"

#ifdef CONFIG_ICD

//#define ICD_INFO_BEFORE_TEST
//#define ICD_EMMC_TEST_DEBUG
//#define ICD_PROTECT_AFTER_TEST

#define LED_ACCESS  0   /* GPIO 83 */
#define LED_REC     1   /* GPIO 84 */

#define EMXX_MMC_OK			0
#define EMXX_MMC_ERR_START		-1
#define EMXX_MMC_ERR_END		-2
#define EMXX_MMC_ERR_PARTITION		-3
#define EMXX_MMC_ERR_PROTECT		-4
#define EMXX_MMC_ERR_WRITE		-5
#define EMXX_MMC_ERR_READ		-6
#define EMXX_MMC_ERR_OTHER		-9

#define EMXX_MMC_ERR_VERIFY		-100

#define LED_BLINK_TIME  (500000)  /* 500ms */

#define SECT_PER_TEST   128
#define TEST_BUF_SIZE   (MMC_BLOCKLEN_VAL*SECT_PER_TEST)

extern int mmc_print_part( unsigned long );
extern int mmc_erase_part( unsigned long );
extern int mmc_protect_part( unsigned long, int );
extern int mmc_write_sect( unsigned long, unsigned long, unsigned long);
extern int mmc_read_sect( unsigned long, unsigned long, unsigned long);
extern int mmc_get_total_block(void);

#ifdef ICD_EMMC_TEST_DEBUG
#define PRINT_INF(fmt, args...)    { printf("%s(): " fmt, __func__, ##args); }
#else
#define PRINT_INF(fmt, args...)
#endif
#define PRINT_ERR(fmt, args...)    { printf("%s(): " fmt, __func__, ##args); }

static char block_w_buf[TEST_BUF_SIZE];
static char block_r_buf[TEST_BUF_SIZE];

static void led_ctrl(int led, int on)
{
    int bit;
    unsigned long reg, val;
    
    switch(led) {
        case LED_ACCESS:
            reg = GIO_064_OH;
#if defined(CONFIG_ICD_0425)
            bit = 8;
#else
            bit = 3;
#endif /* CONFIG_ICD_0425 */
            break;
        case LED_REC:
            reg = GIO_064_OH;
            bit = 4;
            break;
        default:
            PRINT_ERR("%s() unknown led(%d)\n", __func__, led);
            return;
    }
    
    // enable bit set
    val = 1 << (bit + 16);
    
    // output data set
    if(on) {
        val |= 1 << bit;
    }
    
    outl(val, reg);
}

static void led_init(void)
{
#if defined(CONFIG_ICD_0425)
    /* select output gpio 88(ACCESS), 84(REC) */
    outl((inl(CHG_PINSEL_G064) | 0x01100000), CHG_PINSEL_G064);
    outl((inl(CHG_PULL10)      & 0xfff0ffff), CHG_PULL10);
    outl((inl(CHG_PULL11)      & 0xfffffff0), CHG_PULL11);
    outl(0x01100000, GIO_064_E1);
#else    /* select output gpio 83(ACCESS), 84(REC) */
    outl((inl(CHG_PINSEL_G064) | 0x00180000), CHG_PINSEL_G064);
    outl((inl(CHG_PULL10)      & 0xfff00fff), CHG_PULL10);
    outl(0x00180000, GIO_064_E1);
#endif /* CONFIG_ICD_0425 */
    
    /* LEDs on */
    led_ctrl(LED_ACCESS, 1);
    led_ctrl(LED_REC,    1);
}


void err_print(int err)
{
    if(err < 0) {
        switch(err) {
    	case EMXX_MMC_ERR_START:
    			printf("Error: start block not on sector boundary.\n");
    			break;
    	case EMXX_MMC_ERR_END:
    			printf("Error: end block not on sector boundary.\n");
    			break;
    	case EMXX_MMC_ERR_PARTITION:
    			printf("Error: partition is not defined.\n");
    			break;
    	case EMXX_MMC_ERR_PROTECT:
    			printf("Error: protected sectors will not be erased.\n");
    			break;
    	case EMXX_MMC_ERR_OTHER:
    			printf("Error: source address or size is out of DDR area.\n");
    			break;
    	case EMXX_MMC_ERR_VERIFY:
    			printf("Error: verify error.\n");
    			break;
    	default:
    	        printf("Error: erase failed\n");
    			break;
    	}
    }
    else {
        printf("!!! COMPLETED !!!\n");
    }
}

static void test_end(int err)
{
    err_print(err);
    
    while(1) {
        led_ctrl(LED_ACCESS, 0);
        led_ctrl(LED_REC,    0);
        udelay(LED_BLINK_TIME);
        if(err < 0) {
            // if error, blink LED
            led_ctrl(LED_ACCESS, 1);
            led_ctrl(LED_REC,    1);
            udelay(LED_BLINK_TIME);
        }
    }
}

static void err_check(int err)
{
    if(err < 0) {
        test_end(err);
    }
}

static int verify_sect(int blk)
{
    int ret;
    long *p, *p_tail;
    
    // fill write buffer
    p_tail = (long*)(block_w_buf + TEST_BUF_SIZE);
    for(p = block_w_buf; p < p_tail; p++) {
        *p = blk;
    }
    
    ret = mmc_write_sect(block_w_buf, blk, blk + SECT_PER_TEST);
	PRINT_INF("mmc_write_sect(%d,%d) ret=%d\n", blk, SECT_PER_TEST, ret);
	if(ret < 0) {
        return ret;
    }
	
    ret = mmc_read_sect(block_r_buf, blk, blk + SECT_PER_TEST);
	PRINT_INF("mmc_read_sect(%d,%d) ret=%d\n", blk, SECT_PER_TEST, ret);
	if(ret < 0) {
        return ret;
    }

    // verify read buffer
    p_tail = (long*)(block_r_buf + TEST_BUF_SIZE);
    for(p = block_r_buf; p < p_tail; p++) {
        if(*p != blk) {
            PRINT_ERR("verify error, blk=0x%08x buf=0x%08x\n", blk, *p);
            return EMXX_MMC_ERR_VERIFY;
        }
    }
    
    return EMXX_MMC_OK;
}

void icd_emmc_test_erase(void)
{
    int ret;
    
	led_init();
    
#ifdef ICD_INFO_BEFORE_TEST
    // print info
    ret = mmc_print_part(0);
	PRINT_INF("mmc_print_part(all) ret=%d\n", ret);
	err_check(ret);
#endif

    // protect off
    ret = mmc_protect_part(0, 0);
	PRINT_INF("mmc_erase_part(all,off) ret=%d\n", ret);
	err_check(ret);
    
    // erase
    ret = mmc_erase_part(0);
	PRINT_INF("mmc_erase_part(all) ret=%d\n", ret);
	err_check(ret);

#ifdef ICD_PROTECT_AFTER_TEST
    // protect on
    ret = mmc_protect_part(0, 1);
	PRINT_INF("mmc_erase_part(all,on) ret=%d\n", ret);
	err_check(ret);
#endif

    // success
    test_end(0);
}

void icd_emmc_test_verify(void)
{
    int ret, blk, total_blk;
    
	led_init();
    
    // print info
    total_blk = mmc_get_total_block();
	PRINT_INF("mmc_get_total_block() ret=%d\n", total_blk);
    
#ifdef ICD_INFO_BEFORE_TEST
    // print info
    ret = mmc_print_part(0);
	PRINT_INF("mmc_print_part(all) ret=%d\n", ret);
	err_check(ret);
#endif

    // protect off
    ret = mmc_protect_part(0, 0);
	PRINT_INF("mmc_erase_part(all,off) ret=%d\n", ret);
	err_check(ret);
	
    // verify
	for(blk = 0; blk < total_blk; blk += SECT_PER_TEST) {
        ret = verify_sect(blk);
    	PRINT_INF("verify_sect(%d) ret=%d\n", blk, ret);
        err_check(ret);
        printf("ok: %x/%x\n", blk, total_blk);
    }

#ifdef ICD_PROTECT_AFTER_TEST
    // protect on
    ret = mmc_protect_part(0, 1);
	PRINT_INF("mmc_erase_part(all,on) ret=%d\n", ret);
	err_check(ret);
#endif
	
	// success
    test_end(0);
}

#endif
