/* 2013-07-16: File added and changed by Sony Corporation */
/*
 *
 * Copyright 2013 Sony Corporation.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
#include <common.h>

#if defined(CONFIG_ICD_0425)

#include <asm/errno.h>

#include "reg.h"
#include "icx_io.h"
#include "icx_smu.h"
#include "icx_gpio.h"
#include "icx_spi.h"
#include "icx_pwc.h"
#include "icx_pwm.h"
#include "icx_lcdc.h"
#include "icx_lcd.h"
#include "icx_pmu.h"
#include "icx_nvp.h"


/*****************/
/*@ LCDC setting */
/*****************/

/* h/v sync */
#define SITRONIX_ST7586S_HAREA		208
#define SITRONIX_ST7586S_VAREA		145

/* h/v blank */
#define SITRONIX_ST7586S_HBLANK		174
#define SITRONIX_ST7586S_VBLANK		 15

/* h/v total */
#define SITRONIX_ST7586S_HTOTAL		384		/* HAREA + HBLANK */
#define SITRONIX_ST7586S_VTOTAL		160		/* VAREA + VBLANK */

/* conversion macro */
#define COM_REVISE(c)				((c + 2) / 3)


/******************/
/*@ Intervals[ms] */
/******************/

/*  power on */
#define SITRONIX_ST7586S_WAIT_POWER_STABLE				( 50)
#define SITRONIX_ST7586S_WAIT_UNRESET_DEVICE			( 10)
#define SITRONIX_ST7586S_WAIT_UP_LOAD					( 10)
#define SITRONIX_ST7586S_WAIT_CONTROL_OUT				( 20)
#define SITRONIX_ST7586S_WAIT_DISPLAY_SETTING			( 50)

/* power off */
#define SITRONIX_ST7586S_WAIT_RESET_DEVICE				(120)
#define SITRONIX_ST7586S_WAIT_REG_OFF					(200)


/****************/
/*@ LCDC config */
/****************/
const lcdc_config_t lcd_st7586s_config = {
	.width		= SITRONIX_ST7586S_HAREA,
	.height		= SITRONIX_ST7586S_VAREA,
	.frameadr	= LCDC_FRAMEADR_DEFAULT,
	.areaadr	= LCDC_AREAADR_DEFAULT,

	.backlight	= {
		.dev = PWM_DEV_NONE,
	},
};


/***************/
/*@ SPI config */
/***************/

#define SITRONIX_ST7586S_SPI_BLKSIZE		  	  1	/* Block Size		*/

/* SPI config */
static spi_config_t spi_config = {
	.dev    = SPI_DEV_SP3,
	.clk    = SPI_CLK_10MHZ,
	.cs		= SPI_CS_SEL_CS0,
	.pol    = SPI_CK_DLY_OFF | SPI_CK_POL_NEG | SPI_CS_POL_NEG,
	.nbw	= SPI_NB_8BIT,
	.nbr	= 0,
};

/* SPI start byte */
#define SITRONIX_ST7586S_CMD			0x00	/* Set Command						   */
#define SITRONIX_ST7586S_DATA			0x01	/* Set Parameters/Data				   */

/* command */ 									/*     command(with parameters)[ 7: 0] */
												/* ----------------------------------- */
#define SITRONIX_ST7586S_CMD_PS			0x10	/* Power save               SLP[ 0: 0] */
#define SITRONIX_ST7586S_CMD_ID			0x20	/* Inverse Display          INV[ 0: 0] */
#define SITRONIX_ST7586S_CMD_DISP		0x28	/* Display ON/OFF           DSP[ 0: 0] */
#define SITRONIX_ST7586S_CMD_SCA		0x2A	/* Set Column Address                  */
#define SITRONIX_ST7586S_CMD_SRA		0x2B	/* Set Row Address                     */
#define SITRONIX_ST7586S_CMD_WDA		0x2C	/* Write Display Area                  */
#define SITRONIX_ST7586S_CMD_DC			0x36	/* Display Control                     */
#define SITRONIX_ST7586S_CMD_SL			0x37	/* Start Line                          */
#define SITRONIX_ST7586S_CMD_DM			0x38	/* Display Mode               M[ 0: 0] */
#define SITRONIX_ST7586S_CMD_DDRAM		0x3A	/* Enable DDRAM Interface	           */
#define SITRONIX_ST7586S_CMD_DD			0xB0	/* Display Duty                        */
#define SITRONIX_ST7586S_CMD_FOC		0xB1	/* First Output COM                    */
#define SITRONIX_ST7586S_CMD_NLI		0xB5	/* N-Line Inversion                    */
#define SITRONIX_ST7586S_CMD_VOP		0xC0	/* Set Vop                             */
#define SITRONIX_ST7586S_CMD_BS			0xC3	/* BIAS System                         */
#define SITRONIX_ST7586S_CMD_BL			0xC4	/* Booster Level                       */
#define SITRONIX_ST7586S_CMD_VOP_OFST	0xC7	/* Set Vop Offset                      */
#define SITRONIX_ST7586S_CMD_AC			0xD0	/* Analog Control                      */
#define SITRONIX_ST7586S_CMD_ARC		0xD7	/* Auto Read Control		           */
#define SITRONIX_ST7586S_CMD_OTP_RW		0xE0	/* OTP WR/RD Control		           */
#define SITRONIX_ST7586S_CMD_OTP_OUT	0xE1	/* OTP Control Out		               */
#define SITRONIX_ST7586S_CMD_OTP_RD		0xE3	/* OTP Read		                       */
#define SITRONIX_ST7586S_CMD_FR			0xF0	/* Frame Rate		                   */
#define SITRONIX_ST7586S_CMD_TGC		0xF4	/* Temperature Gradient Compensation   */

/* wait command */
#define	SITRONIX_ST7586S_CMD_WAIT		0xFF	/* local rule						   */

#define	SITRONIX_ST7586S_SPI_PARAM_MAX_NUM	8

/* command type */
typedef struct lcd_st7586s_command {
	unsigned char cmd;
	unsigned char param_num;
	unsigned char param[SITRONIX_ST7586S_SPI_PARAM_MAX_NUM];
} lcd_st7586s_command_t;

/* SPI command */
static lcd_st7586s_command_t spi_cmd_power_on[] = {						/* -- Power ON sequence	------ */
	{ SITRONIX_ST7586S_CMD_ARC,          1, {0x9F                  }},	/* Auto Read Control = Disable */
	{ SITRONIX_ST7586S_CMD_OTP_RW,       1, {0x00                  }},	/* OTP WR/RD Control		   */
	{ SITRONIX_ST7586S_CMD_WAIT,         0, {SITRONIX_ST7586S_WAIT_UP_LOAD}},
	{ SITRONIX_ST7586S_CMD_OTP_RD,       0, {0x00                  }},	/* OTP Read		               */
	{ SITRONIX_ST7586S_CMD_WAIT,         0, {SITRONIX_ST7586S_WAIT_CONTROL_OUT}},
	{ SITRONIX_ST7586S_CMD_OTP_OUT,      0, {0x00                  }},	/* OTP Control Out		       */
	{(SITRONIX_ST7586S_CMD_PS   | 0x01), 0, {0x00		           }},	/* Power Save = Sleep out      */
	{(SITRONIX_ST7586S_CMD_DISP | 0x00), 0, {0x00		           }},	/* Display OFF		           */
	{ SITRONIX_ST7586S_CMD_WAIT,         0, {SITRONIX_ST7586S_WAIT_DISPLAY_SETTING}},
	{ SITRONIX_ST7586S_CMD_VOP,          2, {0xF6, 0x00            }},	/* Set Vop		               */
	{ SITRONIX_ST7586S_CMD_BS,           1, {0x04                  }},	/* BIAS System = 1/10		   */
	{ SITRONIX_ST7586S_CMD_BL,           1, {0x07                  }},	/* Booster Level		       */
	{ SITRONIX_ST7586S_CMD_AC,           1, {0x1D                  }},	/* Analog Control		       */
	{ SITRONIX_ST7586S_CMD_NLI,          1, {0x8D                  }},	/* N-Line Inversion		       */
	{(SITRONIX_ST7586S_CMD_DM   | 0x00), 0, {0x00		           }},	/* Display Mode = Gray Mode    */
	{ SITRONIX_ST7586S_CMD_DDRAM,        1, {0x02                  }},	/* 4-level Gray Scale Mode     */
	{ SITRONIX_ST7586S_CMD_DC,           1, {0xC8                  }},	/* Display Control		       */
	{ SITRONIX_ST7586S_CMD_DD,           1, {0x90                  }},	/* Display Duty = 145 duty     */
	{(SITRONIX_ST7586S_CMD_ID   | 0x00), 0, {0x00		           }},	/* Inverse Display		       */
	{ SITRONIX_ST7586S_CMD_FR,           4, {0x12, 0x12, 0x12, 0x12}},	/* Frame Rate		           */
	{ SITRONIX_ST7586S_CMD_TGC,          8, {0x22, 0x00, 0x00, 0x00, 0x00, 0x32, 0x82, 0xB6}},
	{ SITRONIX_ST7586S_CMD_FOC,          1, {0x00                  }},	/* First Output COM		       */
	{ SITRONIX_ST7586S_CMD_SL,           1, {0x00                  }},	/* Start Line		           */
	{(SITRONIX_ST7586S_CMD_DISP | 0x01), 0, {0x00		           }},	/* Display ON		           */
};
#define	SPI_CMD_POWER_ON_NUM	ARRAY_SIZE(spi_cmd_power_on)

static lcd_st7586s_command_t spi_cmd_power_off[] = {               /* -- Power OFF sequence ----- */
	{(SITRONIX_ST7586S_CMD_DISP | 0x00), 0, {0x00                  }},	/* Display OFF				   */
};
#define	SPI_CMD_POWER_OFF_NUM	ARRAY_SIZE(spi_cmd_power_off)

static lcd_st7586s_command_t spi_cmd_contrast[] = {
	{SITRONIX_ST7586S_CMD_VOP_OFST, 1, {0x00}}
};
#define SPI_CMD_CONTRAST_NUM	ARRAY_SIZE(spi_cmd_contrast)


/******************/
/*@ Local routine */
/******************/

/******************************************************************************
 * MODULE   : lcd_st7586s_send_command
 * FUNCTION : send a command & a data 
 * INPUT    : func      : command & data
 *          : num       : the number of data 
 * RETURN   : >=0 : success(the number of send data)
 * NOTE     : none
 * CREATE   : 2013.02.18
 * UPDATE   :
 ******************************************************************************/
static int lcd_st7586s_send_command(lcd_st7586s_command_t *func, int num)
{
	int ret = 0;
	int exec_cmd_count;
	int buf;

	if (NULL == func || 0 == num)
		return 0;

	for (exec_cmd_count = 0; exec_cmd_count < num; ++exec_cmd_count) {

		if (func->cmd == SITRONIX_ST7586S_CMD_WAIT) {
			/* wait */
			mdelay(func->param[0]);
		}
		else {
			int i;

			/* send a command */
			gpio_set_value(GPIO_LCD_CD, SITRONIX_ST7586S_CMD);

			buf = func->cmd;
			ret = spi_write(&spi_config, (u_int *)&buf, SITRONIX_ST7586S_SPI_BLKSIZE);
			if (ret < 0)
				return ret;

			/* send parameters */
			gpio_set_value(GPIO_LCD_CD, SITRONIX_ST7586S_DATA);

			for (i = 0; i < func->param_num; ++i) {
				buf = func->param[i];
				ret = spi_write(&spi_config, (u_int *)&buf, SITRONIX_ST7586S_SPI_BLKSIZE);
				if (ret < 0) {
					exec_cmd_count = ret;
					goto err_cmd;
				}
			}
		}
		/* next command */
		++func;
	}

err_cmd:
	gpio_set_value(GPIO_LCD_CD, SITRONIX_ST7586S_CMD);

	return exec_cmd_count;
}

/******************************************************************************
 * MODULE   : lcd_st7586s_send_image
 * FUNCTION : send a image data 
 * INPUT    : frameadr : address of FB
 *			: rect     : the rectangle updated
 * RETURN   : 0(success)/others(NG)
 * NOTE     : none
 * CREATE   : 2013.02.18
 * UPDATE   :
 ******************************************************************************/
static int lcd_st7586s_send_image(char *frameadr, struct lcdc_rect *rect)
{
	u_int data;
	int size, offset;
	int i;
	int ret  = 0;
	char *src = (char *)0xF0018000;

	lcd_st7586s_command_t spi_cmd_ca_ra[] = {
		{SITRONIX_ST7586S_CMD_SCA, 4, {0x00, COM_REVISE(SITRONIX_ST7586S_HBLANK),
									   0x00, COM_REVISE(SITRONIX_ST7586S_HTOTAL)-1}},
		{SITRONIX_ST7586S_CMD_SRA, 4, {0x00, SITRONIX_ST7586S_VBLANK,
									   0x00, SITRONIX_ST7586S_VTOTAL-1}},
		{SITRONIX_ST7586S_CMD_WDA, 0, {0x00}},
	};

	/* send CA, RA */
	spi_cmd_ca_ra[1].param[1] = SITRONIX_ST7586S_VBLANK + rect->y;
	spi_cmd_ca_ra[1].param[3] = SITRONIX_ST7586S_VBLANK + rect->y + rect->h - 1;
	ret = lcd_st7586s_send_command(spi_cmd_ca_ra, ARRAY_SIZE(spi_cmd_ca_ra));
	if (ret < 0) {
		printf("ST7586S: spi_write error(%d)\n", ret);
		return ret;
	}

	gpio_set_value(GPIO_LCD_CD, SITRONIX_ST7586S_DATA);

	offset = COM_REVISE(SITRONIX_ST7586S_HAREA) * rect->y;
	size   = COM_REVISE(SITRONIX_ST7586S_HAREA) * rect->h;

	/* char to int */
	src += offset;
	for (i = 0; i < size; i++) {
		data = src[i];

		/* send FB data */
		ret = spi_write(&spi_config, &data, 1);
		if (ret < 0) {
			printf("ST7586S: failed to write display data(%d)\n", ret);
			break;
		}
	}

	gpio_set_value(GPIO_LCD_CD, SITRONIX_ST7586S_CMD);

	return ret;
}


/*******************/
/*@ LCDC operation */
/*******************/

/******************************************************************************
 * MODULE   : lcd_st7586s_on
 * FUNCTION : initialize LCD device
 * INPUT    : frameadr : address of FB
 * RETURN   : 0 : success
 * NOTE     : none
 * CREATE   : 2013.02.18
 * UPDATE   :
 ******************************************************************************/
static int lcd_st7586s_on(const lcdc_config_t* config)
{
	int ret = 0;
	struct lcdc_rect rect = {0, 0, SITRONIX_ST7586S_HAREA, SITRONIX_ST7586S_VAREA};
    
   	/* power supply */
   	pmu_regulator_on(REGULATOR_GP4);
	mdelay(SITRONIX_ST7586S_WAIT_POWER_STABLE);

	/* setup spi */
	ret = spi_probe(&spi_config);
	if (ret < 0) {
		printf("LCD on sequence: probe error(%d)\n", ret);
		return ret;
	}

	/* SPI pins set open */
	spi_set_pin_open();

	/* reset LCD device */
	gpio_set_value(GPIO_LCD_XRST, 0x0);
	mdelay(SITRONIX_ST7586S_WAIT_UNRESET_DEVICE);
	gpio_set_value(GPIO_LCD_XRST, 0x1);

	/* send initial code */
	ret = lcd_st7586s_send_command(spi_cmd_power_on, SPI_CMD_POWER_ON_NUM);
	if (ret < 0) {
		printf("LCD on sequence: init error(%d)\n", ret);
		return ret;
	}

	/* send contrast */
	spi_cmd_contrast[0].param[0] = nvp_get_contrast();
	ret = lcd_st7586s_send_command(spi_cmd_contrast, SPI_CMD_CONTRAST_NUM);
	if (ret < 0) {
		printf("LCD on sequence: send contrast error(%d)\n", ret);
		return ret;
	}

	/* send a image */
	ret = lcd_st7586s_send_image((char*)lcd_st7586s_config.frameadr, &rect);
	if (ret < 0) {
		printf("LCD on sequence: send image error(%d)\n", ret);
		return ret;
	}

	/* remove spi */
	spi_remove(&spi_config);

	return 0;
}

/******************************************************************************
 * MODULE   : lcd_st7586s_off
 * FUNCTION : shutdown LCD device
 * INPUT    : none
 * RETURN   : none
 * NOTE     : none
 * CREATE   : 2013.02.18
 * UPDATE   :
 ******************************************************************************/
static void lcd_st7586s_off(void)
{
	spi_probe(&spi_config);

	/* power off the LCD */
	lcd_st7586s_send_command(spi_cmd_power_off, SPI_CMD_POWER_OFF_NUM);
	mdelay(SITRONIX_ST7586S_WAIT_RESET_DEVICE);

	spi_remove(&spi_config);

	/* reset the device */
	gpio_set_value(GPIO_LCD_XRST, 0x0);
	mdelay(SITRONIX_ST7586S_WAIT_REG_OFF);

	/* stop to power supply */
	pmu_regulator_off(REGULATOR_GP4);
}

const lcd_ops_t lcd_st7586s_ops = {
	.power_on  = lcd_st7586s_on,
	.power_off = lcd_st7586s_off,
};

#endif /* CONFIG_ICD_0425 */
